// na mnoha mistech se v handlerech udalosti pouzivaji promenne z jejich lexikalniho uzaveru
// proto se obvykle nesnazim nastavovat obsluhu udalosti jen jednou, ale vsechny handlery
// pokazde nejdriv odstranim a pak nastavim znovu (vyjimkou jsou obsluhy udalosti "novy strom"
// a "novy dokument", registrovane jen jednou pri onReady)

(function() {
    $.couch.uriPrefix = "/couchdb"; // prefix URL, na ktere je CouchDB proxy
    var db = $.couch.db("cms-db"); // databaze, nad kterou CMS bezi

    // trees

    function renewTreesSelect() {
        $("#currentTree, #renameTreeLink, #saveTreeLink, #deleteTreeLink, #showTreeLink").hide();
        db.view("trees/all_trees", {
            success: function(resp) {
                $("#treesSelect").html("").unbind("change");
                for (var row in resp.rows) {
                    var tree = resp.rows[row];
                    $("#treesSelect").append("<option value=\"" + tree.value + "\">" + tree.key + "</option>");
                }
                $("#treesSelect").change(function() {
                    $("option:selected", $(this)).each(function() {
                        showTree($(this));
                    });
                }).change();
            }
        });
    }

    function showTree(selected) {
        var id = $(selected).attr("value");
        db.openDoc(id, {
            success: function(tree) {
                $("#currentTree").show().myTree(tree.content);

                $("#renameTreeLink, #saveTreeLink, #deleteTreeLink, #showTreeLink").show().unbind("click");

                $("#renameTreeLink").click(function() {
                    var oldName = tree.name;
                    jqDialog.prompt("Nové jméno stromu", oldName, function(name) {
                        if (name) {
                            tree.name = name;
                            db.saveDoc(tree, {
                                success: function(resp) {
                                    $.jGrowl("Strom '" + oldName + "' přejmenován na '" + name + "'.");
                                    renewTreesSelect();
                                }
                            });
                        } else {
                            $.jGrowl("Nebylo zadáno jméno!", { header: "Chyba" });
                        }
                    });
                    return false;
                });

                $("#saveTreeLink").click(function() {
                    tree.content = $.tree.reference("#currentTree").get();
                    db.saveDoc(tree, {
                        success: function() {
                            $.jGrowl("Strom '" + tree.name + "' uložen.");
                            renewTreesSelect();
                        }
                    });
                    return false;
                });

                $("#deleteTreeLink").click(function() {
                    db.removeDoc(tree, {
                        success: function() {
                            $.jGrowl("Strom '" + tree.name + "' smazán.");
                            renewTreesSelect();
                        }
                    });
                    return false;
                });

                $("#showTreeLink").attr("href", "http://localhost:5984/cms-db/_design/trees/_show/tree/" + tree._id).attr("target", "_blank");
            }
        });
    }

    // pomocne funkce pro editDoc

    // ve strance existuji vlastne dva stromy: jeden v prehledu dokumentu (ten se da editovat)
    // a druhy v editaci dokumentu (ten ma checkboxy)
    // uzly prvniho stromu maji v DOMu sva ID nastavena primo na databazova ID, takze uzly toho
    // druheho musi mit svoje ID v DOMu odlisna
    // prislusnou zmenu cele stromove struktury provede tato funkce -- pred kazde ID vlozi sluvko "edit"
    function modifyTreeNodesIds(tree) {
        if (tree.attributes.id) {
            tree.attributes.id = "edit" + tree.attributes.id;
        }
        if (tree.children) {
            for (var child in tree.children) {
                modifyTreeNodesIds(tree.children[child]);
            }
        }
    };

    // z pole uzlu stromu v editaci dokumentu (vybranych funkci jQuery) ziska jejich skutecna databazova ID
    // je to vlastne komplement vyse uvedene funkce modifyTreeNodesIds
    function getSelectedTreeNodesIds(nodes) {
        var result = [];
        if (nodes) {
            for (var node in nodes) {
                var id = $(nodes[node]).attr("id").substr(4);
                result.push(id);
            }
        }
        return result;
    }

    // docs

    function renewDocs() {
        db.view("docs/all_docs", {
            success: function(resp) {
                $("#docsTable tbody").html("");
                var counter = 0;
                for (var row in resp.rows) {
                    counter++;
                    addRowInDocsTable(resp.rows[row].value, counter);
                }
            }
        });
    }

    function addRowInDocsTable(doc, counter) {
        var id = doc._id;
        $("#docsTable tbody").append( // TODO osklive, chtelo by to nejake aspon trivialni sablonovani
                "<tr class='" + (counter % 2 == 0 ? "even" : "odd") + "'>" +
                "<td>" + doc.name + "</td><td>" + doc.author + "</td><td>" +
                "<a href='#' id='edit" + id + "'>Upravit</a> " +
                "<a href='#' id='delete" + id + "'>Smazat</a> " +
                "<a href='http://localhost:5984/cms-db/_design/docs/_show/doc/" + id + "' target='_blank'>Zobrazit</a>" +
                "</td></tr>");
        $("#edit" + id).unbind("click").click(function() {
            editDoc(doc);
            return false;
        });
        $("#delete" + id).unbind("click").click(function() {
            db.removeDoc(doc, {
                success: function() {
                    $.jGrowl("Dokument '" + doc.name + "' smazán.");
                    renewDocs();
                }
            });
            return false;
        });
    }

    function editDoc(doc) {
        if (!doc.type) {
            doc.type = "doc";
        }

        $("#docs").hide();
        $("#docEdit").show();

        $("#docEdit input, #docEdit textarea").each(function() {
            var value = doc[$(this).attr("name")];
            $(this).val(value ? value : "");
        });
        $("#docEdit .html textarea").each(function() {
            var value = doc[$(this).attr("name")];
            if ($(this).tinymce()) { // TinyMCE nad jednou textareou nesmim inicializovat vicekrat!
                $(this).html(value ? value : "");
            } else {
                $(this).initMce(value ? value : "");
            }
        });

        var selectedTreeId = $("#treesSelect option:selected").attr("value");
        $("#treesSelect, #currentTree, #treesActions").hide();
        db.openDoc(selectedTreeId, {
            success: function(tree) {
                var treeContent = modifyTreeNodesIds(tree.content);
                $("#editTree").show().checkboxTree(tree.content);
                if (doc.topics) {
                    for (var one in doc.topics) {
                        var topic = doc.topics[one];
                        $.tree.plugins.checkbox.check("#edit" + topic);
                    }
                }
            }
        });

        var goBack = function() {
            $("#docEdit").hide();
            $("#editTree").hide();
            $("#docs").show();
            $("#treesSelect, #currentTree, #treesActions").show();
        };

        $("#saveDocButton, #cancelDocButton").unbind("click");

        $("#saveDocButton").click(function() {
            $("#docEdit input, #docEdit textarea").each(function() {
                doc[$(this).attr("name")] = $(this).val();
            });
            $("#docEdit .html textarea").each(function() {
                doc[$(this).attr("name")] = $(this).html();
            });

            var tree = $.tree.reference("#editTree");
            var checked = $.makeArray($.tree.plugins.checkbox.get_checked(tree));
            doc.topics = getSelectedTreeNodesIds(checked);
            var undetermined = $.makeArray($.tree.plugins.checkbox.get_undeterminded(tree));
            $.merge(checked, undetermined);
            doc.search_topics = getSelectedTreeNodesIds(checked);

            db.saveDoc(doc, {
                success: function() {
                    $.jGrowl("Dokument '" + doc.name + "' uložen.");
                    goBack();
                    renewDocs();
                }
            });
        });

        $("#cancelDocButton").click(goBack);
    }

    // onReady

    $(function() {
        // novy strom
        $("#newTreeLink").click(function() {
            jqDialog.prompt("Nový strom", "", function(name) {
                if (name) {
                    var tree = {
                        type: "tree",
                        name: name,
                        content: {
                            attributes: { id: $.couch.newUUID() },
                            data: name
                        }
                    };
                    db.saveDoc(tree, {
                        success: function(resp) {
                            $.jGrowl("Vytvořen nový strom '" + name + "'.");
                            renewTreesSelect();
                        }
                    });
                } else {
                    $.jGrowl("Nebylo zadáno jméno!", { header: "Chyba" });
                }
            });
            return false;
        });

        // novy dokument
        $("#newDocLink").click(function() {
            editDoc({});
            return false;
        });

        // uvodni obrazovka
        $("#docEdit").hide();
        $("#editTree").hide();
        renewTreesSelect();
        renewDocs();
    });
})();
